@@ -1,11 +1,11 @@
-# $Id: FollowTail.pm,v 1.52 2004/12/02 00:20:58 apocal Exp $
+# $Id: FollowTail.pm,v 1.55 2006/01/25 21:15:21 lotr Exp $
 
 package POE::Wheel::FollowTail;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.52 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my@r=(q$Revision: 1.55 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp qw( croak carp );
 use Symbol qw( gensym );
@@ -58,7 +58,7 @@ sub new {
   my %params = @_;
 
   croak "wheels no longer require a kernel reference as their first parameter"
-    if (@_ && (ref($_[0]) eq 'POE::Kernel'));
+    if @_ and (ref($_[0]) eq 'POE::Kernel');
 
   croak "$type requires a working Kernel" unless (defined $poe_kernel);
 
@@ -76,26 +76,13 @@ sub new {
   my $handle   = $params{Handle};
   my $filename = $params{Filename};
 
-  my @start_stat;
   if (defined $filename) {
-    $handle = gensym();
-
-    # FIFOs (named pipes) are opened R/W so they don't report EOF.
-    # TODO Make this nonfatal, in case the file doesn't exist but will
-    # later.  For example, the file may be caught in the middle of a
-    # rotation.
-    if (-p $filename) {
-      open $handle, "+<$filename" or
-        croak "can't open fifo $filename for R/W: $!";
-    }
-
-    # Everything else is opened read-only.
-    else {
-      open $handle, "<$filename" or croak "can't open $filename: $!";
-    }
-    @start_stat = stat($filename);
+    $handle = _open_file($filename);
   }
 
+  my @start_stat;
+  @start_stat = stat($filename) if defined $handle;
+
   my $poll_interval = (
     defined($params{PollInterval})
     ?  $params{PollInterval}
@@ -116,84 +103,48 @@ sub new {
     $seek = -4096;
   }
 
-  my $self = bless
-    [ $handle,                          # SELF_HANDLE
-      $filename,                        # SELF_FILENAME
-      $driver,                          # SELF_DRIVER
-      $filter,                          # SELF_FILTER
-      $poll_interval,                   # SELF_INTERVAL
-      delete $params{InputEvent},       # SELF_EVENT_INPUT
-      delete $params{ErrorEvent},       # SELF_EVENT_ERROR
-      delete $params{ResetEvent},       # SELF_EVENT_RESET
-      &POE::Wheel::allocate_wheel_id(), # SELF_UNIQUE_ID
-      undef,                            # SELF_STATE_READ
-      \@start_stat,                     # SELF_LAST_STAT
-      undef,                            # SELF_FOLLOW_MODE
-    ], $type;
+  my $self = bless [
+    $handle,                          # SELF_HANDLE
+    $filename,                        # SELF_FILENAME
+    $driver,                          # SELF_DRIVER
+    $filter,                          # SELF_FILTER
+    $poll_interval,                   # SELF_INTERVAL
+    delete $params{InputEvent},       # SELF_EVENT_INPUT
+    delete $params{ErrorEvent},       # SELF_EVENT_ERROR
+    delete $params{ResetEvent},       # SELF_EVENT_RESET
+    &POE::Wheel::allocate_wheel_id(), # SELF_UNIQUE_ID
+    undef,                            # SELF_STATE_READ
+    \@start_stat,                     # SELF_LAST_STAT
+    undef,                            # SELF_FOLLOW_MODE
+  ], $type;
 
-  # SeekBack and partial-input discarding only work for plain files.
-  # SeekBack attempts to position the file pointer somewhere before
-  # the end of the file.  If it's specified, we assume the user knows
-  # where a record begins.  Otherwise we just seek back and discard
-  # everything to EOF so we can frame the input record.
+  # We couldn't open a file.  SeekBack won't be used because it
+  # assumes the file already exists.  If you need more complex
+  # seeking, consider opening and seeking yourself.  In fact, the
+  # whole SeekBack concept should be deprecated as it only opens a can
+  # full of arbitrarily complex worms.  Maybe later.
 
-  if (-f $handle) {
-    my $end = sysseek($handle, 0, SEEK_END);
+  unless (defined $handle) {
+    carp "FollowTail does not support SeekBack on nonexistent files"
+      if defined $params{SeekBack};
 
-    # Seeking back from EOF.
-    if ($seek < 0) {
-      if (defined($end) and ($end < -$seek)) {
-        sysseek($handle, 0, SEEK_SET);
-      }
-      else {
-        sysseek($handle, $seek, SEEK_END);
-      }
-    }
-
-    # Seeking forward from the beginning of the file.
-    elsif ($seek > 0) {
-      if ($seek > $end) {
-        sysseek($handle, 0, SEEK_END);
-      }
-      else {
-        sysseek($handle, $seek, SEEK_SET);
-      }
-    }
-
-    # If they set Seek to 0, we start at the beginning of the file.
-    # If it was SeekBack, we start at the end.
-    elsif (exists $params{Seek}) {
-      sysseek($handle, 0, SEEK_SET);
-    }
-    elsif (exists $params{SeekBack}) {
-      sysseek($handle, 0, SEEK_END);
-    }
-    else {
-      die;
-    }
-
-    # Discard partial input chunks unless a SeekBack was specified.
-    unless (defined $params{SeekBack} or defined $params{Seek}) {
-      while (defined(my $raw_input = $driver->get($handle))) {
-        # Skip out if there's no more input.
-        last unless @$raw_input;
-        $filter->get($raw_input);
-      }
-    }
-
-    # Start the timer loop.
     $self->[SELF_FOLLOW_MODE] = MODE_TIMER;
     $self->_define_timer_states();
+
+    return $self;
   }
 
   # Strange things that ought not be tailed?  Directories...
-  elsif (-d $handle) {
-    croak "FollowTail does not accept directories";
+
+  if (-d $handle) {
+    croak "FollowTail does not tail directories";
   }
 
-  # Otherwise it's not a plain file.  We won't honor SeekBack, and we
-  # will use select_read to watch the handle.
-  else {
+  # SeekBack only works with plain files.  We won't honor SeekBack,
+  # and we will use select_read to watch the handle rather than the
+  # polling interval.
+
+  unless (-f $handle) {
     carp "FollowTail does not support SeekBack on a special file"
       if defined $params{SeekBack};
     carp "FollowTail does not need PollInterval for special files"
@@ -202,8 +153,66 @@ sub new {
     # Start the select loop.
     $self->[SELF_FOLLOW_MODE] = MODE_SELECT;
     $self->_define_select_states();
+
+    return $self;
   }
 
+  # We only get this far with plain files that have successfully been
+  # opened at the time the wheel is created.  SeekBack and
+  # partial-input discarding work here.
+  #
+  # SeekBack attempts to position the file pointer somewhere before
+  # the end of the file.  If it's specified, we assume the user knows
+  # where a record begins.  Otherwise we just seek back and discard
+  # everything to EOF so we can frame the input record.
+
+  my $end = sysseek($handle, 0, SEEK_END);
+
+  # Seeking back from EOF.
+  if ($seek < 0) {
+    if (defined($end) and ($end < -$seek)) {
+      sysseek($handle, 0, SEEK_SET);
+    }
+    else {
+      sysseek($handle, $seek, SEEK_END);
+    }
+  }
+
+  # Seeking forward from the beginning of the file.
+  elsif ($seek > 0) {
+    if ($seek > $end) {
+      sysseek($handle, 0, SEEK_END);
+    }
+    else {
+      sysseek($handle, $seek, SEEK_SET);
+    }
+  }
+
+  # If they set Seek to 0, we start at the beginning of the file.
+  # If it was SeekBack, we start at the end.
+  elsif (exists $params{Seek}) {
+    sysseek($handle, 0, SEEK_SET);
+  }
+  elsif (exists $params{SeekBack}) {
+    sysseek($handle, 0, SEEK_END);
+  }
+  else {
+    die;  # Should never happen.
+  }
+
+  # Discard partial input chunks unless a SeekBack was specified.
+  unless (defined $params{SeekBack} or defined $params{Seek}) {
+    while (defined(my $raw_input = $driver->get($handle))) {
+      # Skip out if there's no more input.
+      last unless @$raw_input;
+      $filter->get($raw_input);
+    }
+  }
+
+  # Start the timer loop.
+  $self->[SELF_FOLLOW_MODE] = MODE_TIMER;
+  $self->_define_timer_states();
+
   return $self;
 }
 
@@ -223,48 +232,48 @@ sub _define_select_states {
   my $event_error = \$self->[SELF_EVENT_ERROR];
   my $event_reset = \$self->[SELF_EVENT_RESET];
 
-  TRACE_POLL and warn "defining select state";
+  TRACE_POLL and warn "<poll> defining select state";
 
-  $poe_kernel->state
-    ( $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> select read",
-      sub {
+  $poe_kernel->state(
+    $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> select read",
+    sub {
 
-        # Protects against coredump on older perls.
-        0 && CRIMSON_SCOPE_HACK('<');
+      # Protects against coredump on older perls.
+      0 && CRIMSON_SCOPE_HACK('<');
 
-        # The actual code starts here.
-        my ($k, $ses) = @_[KERNEL, SESSION];
+      # The actual code starts here.
+      my ($k, $ses) = @_[KERNEL, SESSION];
 
-        eval {
-          sysseek($handle, 0, SEEK_CUR);
-        };
-        $! = 0;
+      eval {
+        sysseek($handle, 0, SEEK_CUR);
+      };
+      $! = 0;
 
-        TRACE_POLL and warn time . " read ok";
+      TRACE_POLL and warn "<poll> " . time . " read ok";
 
-        if (defined(my $raw_input = $driver->get($handle))) {
-          if (@$raw_input) {
-            TRACE_POLL and warn time . " raw input";
-            foreach my $cooked_input (@{$filter->get($raw_input)}) {
-              TRACE_POLL and warn time . " cooked input";
-              $k->call($ses, $$event_input, $cooked_input, $unique_id);
-            }
+      if (defined(my $raw_input = $driver->get($handle))) {
+        if (@$raw_input) {
+          TRACE_POLL and warn "<poll> " . time . " raw input";
+          foreach my $cooked_input (@{$filter->get($raw_input)}) {
+            TRACE_POLL and warn "<poll> " . time . " cooked input";
+            $k->call($ses, $$event_input, $cooked_input, $unique_id);
           }
         }
-
-        # Error reading.  Report the error if it's not EOF, or if it's
-        # EOF on a socket or TTY.  Shut down the select, too.
-        else {
-          if ($! or (-S $handle) or (-t $handle)) {
-            TRACE_POLL and warn time . " error: $!";
-            $$event_error and
-              $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
-            $k->select($handle);
-          }
-          eval { IO::Handle::clearerr($handle) }; # could be a globref
-        }
       }
-    );
+
+      # Error reading.  Report the error if it's not EOF, or if it's
+      # EOF on a socket or TTY.  Shut down the select, too.
+      else {
+        if ($! or (-S $handle) or (-t $handle)) {
+          TRACE_POLL and warn "<poll> " . time . " error: $!";
+          $$event_error and
+            $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
+          $k->select($handle);
+        }
+        eval { IO::Handle::clearerr($handle) }; # could be a globref
+      }
+    }
+  );
 
   $poe_kernel->select_read($handle, $self->[SELF_STATE_READ]);
 }
@@ -281,119 +290,124 @@ sub _define_timer_states {
   my $poll_interval = $self->[SELF_INTERVAL];
   my $filename      = $self->[SELF_FILENAME];
   my $last_stat     = $self->[SELF_LAST_STAT];
-  my $handle        = $self->[SELF_HANDLE];
-  my $state_read    = $self->[SELF_STATE_READ] =
-    ref($self) . "($unique_id) -> timer read";
 
+  my $handle        = \$self->[SELF_HANDLE];
   my $event_input   = \$self->[SELF_EVENT_INPUT];
   my $event_error   = \$self->[SELF_EVENT_ERROR];
   my $event_reset   = \$self->[SELF_EVENT_RESET];
 
-  TRACE_POLL and warn "defining timer state";
+  $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> timer read";
+  my $state_read    = \$self->[SELF_STATE_READ];
 
-  $poe_kernel->state
-    ( $state_read,
-      sub {
+  TRACE_POLL and warn "<poll> defining timer state";
 
-        # Protects against coredump on older perls.
-        0 && CRIMSON_SCOPE_HACK('<');
+  $poe_kernel->state(
+    $$state_read,
+    sub {
 
-        # The actual code starts here.
-        my ($k, $ses) = @_[KERNEL, SESSION];
+      # Protects against coredump on older perls.
+      0 && CRIMSON_SCOPE_HACK('<');
 
-        eval {
-          if (defined $filename) {
-            my @new_stat = stat($filename);
+      # The actual code starts here.
+      my ($k, $ses) = @_[KERNEL, SESSION];
 
-            TRACE_STAT_VERBOSE and do {
-              my @test_new = @new_stat;   splice(@test_new, 8, 1, "(removed)");
-              my @test_old = @$last_stat; splice(@test_old, 8, 1, "(removed)");
-              warn "=== @test_new" if "@test_new" ne "@test_old";
-            };
+      eval {
+        if (defined $filename) {
+          my @new_stat = stat($filename);
 
-            if (@new_stat) {
+          TRACE_STAT_VERBOSE and do {
+            my @test_new = @new_stat;   splice(@test_new, 8, 1, "(removed)");
+            my @test_old = @$last_stat; splice(@test_old, 8, 1, "(removed)");
+            warn "<stat> @test_new" if "@test_new" ne "@test_old";
+          };
 
-              # File shrank.  Consider it a reset.  Seek to the top of
-              # the file.
-              if ($new_stat[7] < $last_stat->[7]) {
-                $$event_reset and $k->call($ses, $$event_reset, $unique_id);
-                sysseek($handle, 0, SEEK_SET);
-              }
+          if (@new_stat) {
+            my $did_reset;
 
-              $last_stat->[7] = $new_stat[7];
+            # File shrank.  Consider it a reset.  Seek to the top of
+            # the file.
+            if ($new_stat[7] < $last_stat->[7]) {
+              $did_reset = 1;
+            }
 
-              # Something fundamental about the file changed.  Reopen it.
-              if ( $new_stat[1] != $last_stat->[1] or # inode's number
-                   $new_stat[0] != $last_stat->[0] or # inode's device
-                   $new_stat[6] != $last_stat->[6] or # device type
-                   $new_stat[3] != $last_stat->[3]    # number of links
-                 ) {
+            $last_stat->[7] = $new_stat[7];
 
-                TRACE_STAT and do {
-                  warn "inode $new_stat[1] != old $last_stat->[1]\n"
-                    if $new_stat[1] != $last_stat->[1];
-                  warn "inode device $new_stat[0] != old $last_stat->[0]\n"
-                    if $new_stat[0] != $last_stat->[0];
-                  warn "device type $new_stat[6] != old $last_stat->[6]\n"
-                    if $new_stat[6] != $last_stat->[6];
-                  warn "number of links $new_stat[3] != old $last_stat->[3]\n"
-                    if $new_stat[3] != $last_stat->[3];
-                  warn "file size $new_stat[7] < old $last_stat->[7]\n"
-                    if $new_stat[7] < $last_stat->[7];
-                };
+            # Something fundamental about the file changed.  Reopen it.
+            if (
+              $new_stat[1] != $last_stat->[1] or # inode's number
+              $new_stat[0] != $last_stat->[0] or # inode's device
+              $new_stat[6] != $last_stat->[6] or # device type
+              $new_stat[3] != $last_stat->[3]    # number of links
+            ) {
 
-                @$last_stat = @new_stat;
+              TRACE_STAT and do {
+                warn "<stat> inode $new_stat[1] != old $last_stat->[1]\n"
+                  if $new_stat[1] != $last_stat->[1];
+                warn "<stat> inode device $new_stat[0] != old $last_stat->[0]\n"
+                  if $new_stat[0] != $last_stat->[0];
+                warn "<stat> device type $new_stat[6] != old $last_stat->[6]\n"
+                  if $new_stat[6] != $last_stat->[6];
+                warn "<stat> link count $new_stat[3] != old $last_stat->[3]\n"
+                  if $new_stat[3] != $last_stat->[3];
+                warn "<stat> file size $new_stat[7] < old $last_stat->[7]\n"
+                  if $new_stat[7] < $last_stat->[7];
+              };
 
-                close $handle;
-                unless (open $handle, "<$filename") {
-                  $$event_error and
-                    $k->call( $ses, $$event_error, 'reopen',
-                              ($!+0), $!, $unique_id
-                            );
-                }
-              }
+              @$last_stat = @new_stat;
+              close $$handle if defined $$handle;
+              $$handle = _open_file($filename);
+
+              $did_reset = 1;
+            }
+
+            if ($did_reset) {
+              $$event_reset and $k->call($ses, $$event_reset, $unique_id);
+              sysseek($$handle, 0, SEEK_SET);
             }
           }
-        };
-        $! = 0;
-
-        TRACE_POLL and warn time . " read ok\n";
-
-        # Got input.  Read a bunch of it, then poll again right away.
-        if (defined(my $raw_input = $driver->get($handle))) {
-          if (@$raw_input) {
-            TRACE_POLL and warn time . " raw input\n";
-            foreach my $cooked_input (@{$filter->get($raw_input)}) {
-              TRACE_POLL and warn time . " cooked input\n";
-              $k->call($ses, $$event_input, $cooked_input, $unique_id);
-            }
-          }
-          $k->yield($state_read);
         }
+      };
 
-        # Got an error of some sort.
-        else {
-          TRACE_POLL and warn time . " set delay\n";
-          if ($!) {
-            TRACE_POLL and warn time . " error: $!\n";
-            $$event_error and
-              $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
-            $k->select($handle);
-          }
-          $k->delay($state_read, $poll_interval);
-          IO::Handle::clearerr($handle);
-        }
+      $! = 0;
+      TRACE_POLL and warn "<poll> " . time . " read ok\n";
+
+      # No open file.  Go around again.
+      unless (defined $$handle) {
+        $k->delay($$state_read, $poll_interval) if defined $$state_read;
       }
-    );
 
-  $poe_kernel->yield($state_read);
+      # Got input.  Read a bunch of it, then poll again right away.
+      elsif (defined(my $raw_input = $driver->get($$handle))) {
+        if (@$raw_input) {
+          TRACE_POLL and warn "<poll> " . time . " raw input\n";
+          foreach my $cooked_input (@{$filter->get($raw_input)}) {
+            TRACE_POLL and warn "<poll> " . time . " cooked input\n";
+            $k->call($ses, $$event_input, $cooked_input, $unique_id);
+          }
+        }
+        $k->yield($$state_read) if defined $$state_read;
+      }
+
+      # Got an error of some sort.
+      else {
+        TRACE_POLL and warn "<poll> " . time . " set delay\n";
+        if ($!) {
+          TRACE_POLL and warn "<poll> " . time . " error: $!\n";
+          $$event_error and
+            $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
+          $k->select($$handle);
+        }
+        $k->delay($$state_read, $poll_interval) if defined $$state_read;
+        IO::Handle::clearerr($$handle);
+      }
+    }
+  );
+
+  # Fire up the loop.  The delay() aspect of the loop will prevent
+  # duplicate events from being significant for long.
+  $poe_kernel->delay($$state_read, 0);
 }
 
-# ### Define the select states, and begin reading the special handle.
-# ### This also relies on stupid closure tricks.
-
-#     $poe_kernel->select($handle, $self->[SELF_STATE_READ]);
-
 #------------------------------------------------------------------------------
 
 sub event {
@@ -421,16 +435,6 @@ sub event {
       carp "ignoring unknown FollowTail parameter '$name'";
     }
   }
-
-  if ($self->[SELF_FOLLOW_MODE] & MODE_TIMER) {
-    $self->_define_timer_states();
-  }
-  elsif ($self->[SELF_FOLLOW_MODE] & MODE_SELECT) {
-    $self->_define_select_states();
-  }
-  else {
-    die;
-  }
 }
 
 #------------------------------------------------------------------------------
@@ -439,7 +443,8 @@ sub DESTROY {
   my $self = shift;
 
   # Remove our tentacles from our owner.
-  $poe_kernel->select($self->[SELF_HANDLE]);
+  $poe_kernel->select($self->[SELF_HANDLE]) if defined $self->[SELF_HANDLE];
+
   $poe_kernel->delay($self->[SELF_STATE_READ]);
 
   if ($self->[SELF_STATE_READ]) {
@@ -456,6 +461,28 @@ sub ID {
   return $_[0]->[SELF_UNIQUE_ID];
 }
 
+sub tell {
+  my $self = shift;
+  return sysseek($self->[SELF_HANDLE], 0, SEEK_CUR);
+}
+
+sub _open_file {
+  my $filename = shift;
+
+  my $handle = gensym();
+
+  # FIFOs (named pipes) are opened R/W so they don't report EOF.
+  # Everything else is opened read-only.
+  if (-p $filename) {
+    return unless open($handle, "+<$filename");
+  }
+  else {
+    return unless open($handle, "<$filename");
+  }
+
+  return $handle;
+}
+
 ###############################################################################
 1;
 
@@ -491,6 +518,8 @@ POE::Wheel::FollowTail - follow the tail of an ever-growing file
     Seek         => $offset,            # How far from BOF to start
   );
 
+  $pos = $wheel->tell();  # Get the current log position.
+
 =head1 DESCRIPTION
 
 FollowTail follows the end of an ever-growing file, such as a log of
@@ -516,6 +545,18 @@ The ID method returns a FollowTail wheel's unique ID.  This ID will be
 included in every event the wheel generates, and it can be used to
 match events with the wheels which generated them.
 
+=item tell
+
+Returns where POE::Wheel::FollowTail is currently in the log file.
+tell() may be useful for seeking back into a file when resuming
+tailing.
+
+  my $pos = $wheel->tell();
+
+FollowTail generally reads ahead of the data it returns, so the file
+position is not necessarily the point after the last record you have
+received.
+
 =back
 
 =head1 EVENTS AND PARAMETERS
@@ -605,6 +646,8 @@ FollowTail can watch a file or device that's already open.  Give it
 the open filehandle with its Handle parameter.
 
 FollowTail can watch a file by name, given as the Filename parameter.
+The named file does not need to exist.  FollowTail will wait for it to
+appear.
 
 This wheel can detect files that have been "reset".  That is, it can
 tell when log files have been restarted due to a rotation or purge.
@@ -676,10 +719,10 @@ just prior to a file reset.  For a more robust way to watch files,
 consider using POE::Wheel::Run and your operating system's native
 "tail" utility instead.
 
-  $heap->{tail} = POE::Wheel::Run->new
-    ( Program     => [ "/usr/bin/tail", "-f", $file_name ],
-      StdoutEvent => "log_record",
-    );
+  $heap->{tail} = POE::Wheel::Run->new(
+    Program     => [ "/usr/bin/tail", "-f", $file_name ],
+    StdoutEvent => "log_record",
+  );
 
 =head1 AUTHORS & COPYRIGHTS
 
