@@ -1,11 +1,11 @@
-# $Id: Run.pm,v 1.69 2005/04/24 15:58:40 rcaputo Exp $
+# $Id: Run.pm,v 1.72 2006/01/26 05:59:50 rcaputo Exp $
 
 package POE::Wheel::Run;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.69 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my@r=(q$Revision: 1.72 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -27,13 +27,13 @@ BEGIN {
   }
 
   if (POE::Kernel::RUNNING_IN_HELL) {
-      eval    { require Win32::Console; };
-      if ($@) { die "Win32::Console failed to load:\n$@" }
-      else    { Win32::Console->import(); };
+    eval    { require Win32::Console; };
+    if ($@) { die "Win32::Console failed to load:\n$@" }
+    else    { Win32::Console->import(); };
 
-      eval    { require Win32API::File; };
-      if ($@) { die "Win32API::File but failed to load:\n$@" }
-      else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
+    eval    { require Win32API::File; };
+    if ($@) { die "Win32API::File but failed to load:\n$@" }
+    else    { Win32API::File->import( qw(FdGetOsFHandle) ); };
   }
 
   # How else can I get them out?!
@@ -148,13 +148,12 @@ sub new {
   my $conduit = delete $params{Conduit};
   my $stdio_type;
   if (defined $conduit) {
-    croak "$type\'s Conduit type ($conduit) is unknown"
-      if (
-        $conduit ne 'pipe' and
-        $conduit ne 'pty'  and
-        $conduit ne 'socketpair' and
-        $conduit ne 'inet'
-      );
+    croak "$type\'s Conduit type ($conduit) is unknown" if (
+      $conduit ne 'pipe' and
+      $conduit ne 'pty'  and
+      $conduit ne 'socketpair' and
+      $conduit ne 'inet'
+    );
     unless ($conduit eq "pty") {
       $stdio_type = $conduit;
       $conduit = "pipe";
@@ -478,20 +477,20 @@ sub new {
       # Give up with a plain exit if we must.
       # On win32 cannot _exit as it will kill *all* threads, meaning parent too
       unless (POE::Kernel::RUNNING_IN_HELL) {
-    eval { POSIX::_exit(0);  };
-    eval { kill KILL => $$;  };
-    eval { exec("$^X -e 0"); };
+        eval { POSIX::_exit(0);  };
+        eval { kill KILL => $$;  };
+        eval { exec("$^X -e 0"); };
       };
       exit(0);
     } else {
-  if (ref($program) eq 'ARRAY') {
-    exec(@$program, @$prog_args)
-      or die "can't exec (@$program) in child pid $$: $!";
-  }
-  else {
-    exec(join(" ", $program, @$prog_args))
-      or die "can't exec ($program) in child pid $$: $!";
-  }
+      if (ref($program) eq 'ARRAY') {
+        exec(@$program, @$prog_args)
+          or die "can't exec (@$program) in child pid $$: $!";
+      }
+      else {
+        exec(join(" ", $program, @$prog_args))
+          or die "can't exec ($program) in child pid $$: $!";
+      }
     }
     die "insanity check passed";
   }
@@ -618,97 +617,99 @@ sub _define_stdin_flusher {
 sub _define_stdout_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDOUT];
+
+  # No event?  Unregister the handler and leave.
+  my $stdout_event  = \$self->[EVENT_STDOUT];
+  unless ($$stdout_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDOUT]);
+    return;
+  }
+
+  # If any of these change, then the read state is invalidated and
+  # needs to be redefined.
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDOUT];
+  my $stdout_filter = $self->[FILTER_STDOUT];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $is_active     = \$self->[IS_ACTIVE];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $error_event   = \$self->[ERROR_EVENT];
+
   # Register the select-read handler for STDOUT.
-  if (defined $self->[HANDLE_STDOUT]) {
+  if (
+    $stdout_filter->can("get_one") and
+    $stdout_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDOUT];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stdout_filter = $self->[FILTER_STDOUT];
-    my $stdout_event  = \$self->[EVENT_STDOUT];
-    my $is_active     = \$self->[IS_ACTIVE];
-
-    if (
-      $stdout_filter->can("get_one") and
-      $stdout_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stdout_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stdout_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stdout_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDOUT'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get one.
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stdout_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stdout_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stdout_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDOUT'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDOUT'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get one.
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDOUT] = ref($self) . "($unique_id) -> select stdout",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stdout_filter->get($raw_input)}) {
+            $k->call($me, $$stdout_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
+        else {
+          $$error_event and
+            $k->call(
+              $me, $$error_event,
+              'read', ($!+0), $!, $unique_id, 'STDOUT'
+            );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDOUT.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDOUT])
-      if defined $self->[HANDLE_STDOUT];
-  }
+  # register the state's select
+  $poe_kernel->select_read($self->[HANDLE_STDOUT], $self->[STATE_STDOUT]);
 }
 
 #------------------------------------------------------------------------------
@@ -718,96 +719,96 @@ sub _define_stdout_reader {
 sub _define_stderr_reader {
   my $self = shift;
 
+  # Can't do anything if we don't have a handle.
+  return unless defined $self->[HANDLE_STDERR];
+
+  # No event?  Unregister the handler and leave.
+  my $stderr_event  = \$self->[EVENT_STDERR];
+  unless ($$stderr_event) {
+    $poe_kernel->select_read($self->[HANDLE_STDERR]);
+    return;
+  }
+
+  my $unique_id     = $self->[UNIQUE_ID];
+  my $driver        = $self->[DRIVER_STDERR];
+  my $stderr_filter = $self->[FILTER_STDERR];
+
+  # These can change without redefining the callback since they're
+  # enclosed by reference.
+  my $error_event   = \$self->[ERROR_EVENT];
+  my $close_event   = \$self->[CLOSE_EVENT];
+  my $is_active     = \$self->[IS_ACTIVE];
+
   # Register the select-read handler for STDERR.
-  if (defined $self->[HANDLE_STDERR]) {
-    # If any of these change, then the read state is invalidated and
-    # needs to be redefined.
-    my $unique_id     = $self->[UNIQUE_ID];
-    my $driver        = $self->[DRIVER_STDERR];
-    my $error_event   = \$self->[ERROR_EVENT];
-    my $close_event   = \$self->[CLOSE_EVENT];
-    my $stderr_filter = $self->[FILTER_STDERR];
-    my $stderr_event  = \$self->[EVENT_STDERR];
-    my $is_active     = \$self->[IS_ACTIVE];
+  if (
+    $stderr_filter->can("get_one") and
+    $stderr_filter->can("get_one_start")
+  ) {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
 
-    if (
-      $stderr_filter->can("get_one") and
-      $stderr_filter->can("get_one_start")
-    ) {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            $stderr_filter->get_one_start($raw_input);
-            while (1) {
-              my $next_rec = $stderr_filter->get_one();
-              last unless @$next_rec;
-              foreach my $cooked_input (@$next_rec) {
-                $k->call($me, $$stderr_event, $cooked_input, $unique_id);
-              }
-            }
-          }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDERR'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
-          }
-        }
-      );
-    }
-
-    # Otherwise we can't get_one().
-    else {
-      $poe_kernel->state(
-        $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
-        sub {
-          # prevents SEGV
-          0 && CRIMSON_SCOPE_HACK('<');
-
-          # subroutine starts here
-          my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
-          if (defined(my $raw_input = $driver->get($handle))) {
-            foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          $stderr_filter->get_one_start($raw_input);
+          while (1) {
+            my $next_rec = $stderr_filter->get_one();
+            last unless @$next_rec;
+            foreach my $cooked_input (@$next_rec) {
               $k->call($me, $$stderr_event, $cooked_input, $unique_id);
             }
           }
-          else {
-            $$error_event and
-              $k->call(
-                $me, $$error_event,
-                'read', ($!+0), $!, $unique_id, 'STDERR'
-              );
-            unless (--$$is_active) {
-              $k->call( $me, $$close_event, $unique_id )
-                if defined $$close_event;
-            }
-            $k->select_read($handle);
+        }
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
+  }
+
+  # Otherwise we can't get_one().
+  else {
+    $poe_kernel->state(
+      $self->[STATE_STDERR] = ref($self) . "($unique_id) -> select stderr",
+      sub {
+        # prevents SEGV
+        0 && CRIMSON_SCOPE_HACK('<');
+
+        # subroutine starts here
+        my ($k, $me, $handle) = @_[KERNEL, SESSION, ARG0];
+        if (defined(my $raw_input = $driver->get($handle))) {
+          foreach my $cooked_input (@{$stderr_filter->get($raw_input)}) {
+            $k->call($me, $$stderr_event, $cooked_input, $unique_id);
           }
         }
-      );
-    }
-
-    # register the state's select
-    $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
+        else {
+          $$error_event and $k->call(
+            $me, $$error_event,
+            'read', ($!+0), $!, $unique_id, 'STDERR'
+          );
+          unless (--$$is_active) {
+            $k->call( $me, $$close_event, $unique_id )
+              if defined $$close_event;
+          }
+          $k->select_read($handle);
+        }
+      }
+    );
   }
 
-  # Register the select-read handler for STDERR.
-  else {
-    $poe_kernel->select_read($self->[HANDLE_STDERR])
-      if defined $self->[HANDLE_STDERR];
-  }
+  # Register the state's select.
+  $poe_kernel->select_read($self->[HANDLE_STDERR], $self->[STATE_STDERR]);
 }
 
 #------------------------------------------------------------------------------
@@ -849,10 +850,6 @@ sub event {
       carp "ignoring unknown Run parameter '$name'";
     }
   }
-
-  $self->_define_stdin_flusher() if defined $redefine_stdin;
-  $self->_define_stdout_reader() if defined $redefine_stdout;
-  $self->_define_stderr_reader() if defined $redefine_stderr;
 }
 
 #------------------------------------------------------------------------------
@@ -1053,7 +1050,6 @@ sub set_stdout_filter {
   my $buf = $self->[FILTER_STDOUT]->get_pending();
   $self->[FILTER_STDOUT] = $new_filter;
 
-  $self->_define_stdout_reader();
   $self->_transfer_stdout_buffer($buf);
 }
 
@@ -1063,7 +1059,6 @@ sub set_stderr_filter {
   my $buf = $self->[FILTER_STDERR]->get_pending();
   $self->[FILTER_STDERR] = $new_filter;
 
-  $self->_define_stderr_reader();
   $self->_transfer_stderr_buffer($buf);
 }
 
